@@ -13,9 +13,10 @@ import numbers
 
 from .splitting import Splitter
 from .histogram import HistogramBuilder
-from .predictor import TreePredictor, PREDICTOR_RECORD_DTYPE
+from .predictor import TreePredictor
 from .utils import sum_parallel
-from .types import Y_DTYPE
+from .common import PREDICTOR_RECORD_DTYPE
+from .common import Y_DTYPE
 
 
 EPS = np.finfo(Y_DTYPE).eps  # to avoid zero division errors
@@ -140,14 +141,18 @@ class TreeGrower:
     min_gain_to_split : float, optional (default=0.)
         The minimum gain needed to split a node. Splits with lower gain will
         be ignored.
-    max_bins : int, optional (default=256)
-        The maximum number of bins. Used to define the shape of the
-        histograms.
-    actual_n_bins : ndarray of int or int, optional (default=None)
-        The actual number of bins needed for each feature, which is lower or
-        equal to ``max_bins``. If it's an int, all features are considered to
-        have the same number of bins. If None, all features are considered to
-        have ``max_bins`` bins.
+    n_bins : int, optional (default=256)
+        The total number of bins, including the bin for missing values. Used
+        to define the shape of the histograms.
+    n_bins_non_missing_ : array of uint32
+        For each feature, gives the number of bins actually used for
+        non-missing values. For features with a lot of unique values, this
+        is equal to ``n_bins - 1``. If it's an int, all features are
+        considered to have the same number of bins. If None, all features
+        are considered to have ``n_bins - 1`` bins.
+    has_missing_values : ndarray of bool or bool, optional (default=False)
+        Whether each feature contains missing values (in the training data).
+        If it's a bool, the same value is used for all features.
     l2_regularization : float, optional (default=0)
         The L2 regularization parameter.
     min_hessian_to_split : float, optional (default=1e-3)
@@ -160,32 +165,40 @@ class TreeGrower:
     """
     def __init__(self, X_binned, gradients, hessians, max_leaf_nodes=None,
                  max_depth=None, min_samples_leaf=20, min_gain_to_split=0.,
-                 max_bins=256, actual_n_bins=None, l2_regularization=0.,
-                 min_hessian_to_split=1e-3, shrinkage=1.):
+                 n_bins=256, n_bins_non_missing=None, has_missing_values=False,
+                 l2_regularization=0., min_hessian_to_split=1e-3,
+                 shrinkage=1.):
 
         self._validate_parameters(X_binned, max_leaf_nodes, max_depth,
                                   min_samples_leaf, min_gain_to_split,
                                   l2_regularization, min_hessian_to_split)
 
-        if actual_n_bins is None:
-            actual_n_bins = max_bins
+        if n_bins_non_missing is None:
+            n_bins_non_missing = n_bins - 1
 
-        if isinstance(actual_n_bins, numbers.Integral):
-            actual_n_bins = np.array(
-                [actual_n_bins] * X_binned.shape[1],
+        if isinstance(n_bins_non_missing, numbers.Integral):
+            n_bins_non_missing = np.array(
+                [n_bins_non_missing] * X_binned.shape[1],
                 dtype=np.uint32)
         else:
-            actual_n_bins = np.asarray(actual_n_bins, dtype=np.uint32)
+            n_bins_non_missing = np.asarray(n_bins_non_missing,
+                                            dtype=np.uint32)
+
+        if isinstance(has_missing_values, bool):
+            has_missing_values = [has_missing_values] * X_binned.shape[1]
+        has_missing_values = np.asarray(has_missing_values, dtype=np.uint8)
 
         hessians_are_constant = hessians.shape[0] == 1
         self.histogram_builder = HistogramBuilder(
-            X_binned, max_bins, gradients, hessians, hessians_are_constant)
+            X_binned, n_bins, gradients, hessians, hessians_are_constant)
+        missing_values_bin_idx = n_bins - 1
         self.splitter = Splitter(
-            X_binned, max_bins, actual_n_bins, l2_regularization,
-            min_hessian_to_split, min_samples_leaf, min_gain_to_split,
-            hessians_are_constant)
+            X_binned, n_bins_non_missing, missing_values_bin_idx,
+            has_missing_values, l2_regularization, min_hessian_to_split,
+            min_samples_leaf, min_gain_to_split, hessians_are_constant)
+        self.n_bins_non_missing = n_bins_non_missing
         self.max_leaf_nodes = max_leaf_nodes
-        self.max_bins = max_bins
+        self.has_missing_values = has_missing_values
         self.n_features = X_binned.shape[1]
         self.max_depth = max_depth
         self.min_samples_leaf = min_samples_leaf
@@ -279,7 +292,7 @@ class TreeGrower:
         """
 
         node.split_info = self.splitter.find_node_split(
-            node.sample_indices, node.histograms, node.sum_gradients,
+            node.n_samples, node.histograms, node.sum_gradients,
             node.sum_hessians)
 
         if node.split_info.gain <= 0:  # no valid split
@@ -332,6 +345,13 @@ class TreeGrower:
         right_child_node.partition_start = left_child_node.partition_stop
         right_child_node.partition_stop = node.partition_stop
 
+        if not self.has_missing_values[node.split_info.feature_idx]:
+            # If no missing values are encountered at fit time, then samples
+            # with missing values during predict() will go to whichever child
+            # has the most samples.
+            node.split_info.missing_go_to_left = (
+                left_child_node.n_samples > right_child_node.n_samples)
+
         self.n_nodes += 2
 
         if self.max_depth is not None and depth == self.max_depth:
@@ -427,12 +447,13 @@ class TreeGrower:
         """
         predictor_nodes = np.zeros(self.n_nodes, dtype=PREDICTOR_RECORD_DTYPE)
         _fill_predictor_node_array(predictor_nodes, self.root,
-                                   bin_thresholds=bin_thresholds)
+                                   bin_thresholds, self.n_bins_non_missing)
         return TreePredictor(predictor_nodes)
 
 
 def _fill_predictor_node_array(predictor_nodes, grower_node,
-                               bin_thresholds, next_free_idx=0):
+                               bin_thresholds, n_bins_non_missing,
+                               next_free_idx=0):
     """Helper used in make_predictor to set the TreePredictor fields."""
     node = predictor_nodes[next_free_idx]
     node['count'] = grower_node.n_samples
@@ -453,17 +474,27 @@ def _fill_predictor_node_array(predictor_nodes, grower_node,
         feature_idx, bin_idx = split_info.feature_idx, split_info.bin_idx
         node['feature_idx'] = feature_idx
         node['bin_threshold'] = bin_idx
-        if bin_thresholds is not None:
-            threshold = bin_thresholds[feature_idx][bin_idx]
-            node['threshold'] = threshold
+        node['missing_go_to_left'] = split_info.missing_go_to_left
+
+        if split_info.bin_idx == n_bins_non_missing[feature_idx] - 1:
+            # Split is on the last non-missing bin: it's a "split on nans". All
+            # nans go to the right, the rest go to the left.
+            node['threshold'] = np.inf
+        elif bin_thresholds is not None:
+            node['threshold'] = bin_thresholds[feature_idx][bin_idx]
+
         next_free_idx += 1
 
         node['left'] = next_free_idx
         next_free_idx = _fill_predictor_node_array(
             predictor_nodes, grower_node.left_child,
-            bin_thresholds=bin_thresholds, next_free_idx=next_free_idx)
+            bin_thresholds=bin_thresholds,
+            n_bins_non_missing=n_bins_non_missing,
+            next_free_idx=next_free_idx)
 
         node['right'] = next_free_idx
         return _fill_predictor_node_array(
             predictor_nodes, grower_node.right_child,
-            bin_thresholds=bin_thresholds, next_free_idx=next_free_idx)
+            bin_thresholds=bin_thresholds,
+            n_bins_non_missing=n_bins_non_missing,
+            next_free_idx=next_free_idx)
